clear
clc
warning('off','all')

mkdir('temp');
mkdir('results');

load('CHdata.mat');

SampleSelect = inc>=10000 & inc<=200000;
n = sum(SampleSelect);

Y = net_tfa(SampleSelect)/1000; 
T = p401(SampleSelect);
Z = e401(SampleSelect);
V = [inc/1000,age-25,(age-25).^2,marr,fsize];
V = V(SampleSelect,:);
R = pira(SampleSelect);

N_boot  = 2000;
pi_grid = 50;
method_list = 1:3;
K_list = 1:8;
N_para = size(method_list,2)*size(K_list,2);

beta   = .001;
delta  = .01;
alpha  = .05-delta;

epsilon_boot = norminv(rand(n,N_boot));

UB = max(Y)-min(Y);
LB = -UB;

[b,dev,stats] = glmfit(V,Z,'normal');
Pi = glmval(b,V,'identity',stats);

W_boot = NaN(n,pi_grid);
for pi_index = 1:pi_grid
    
    temp = randn(size(b));    
    Pi = glmval(b+chol(stats.covb,'lower')*temp*sqrt(chi2cdf(1-delta,size(b,1))/(temp'*temp)),V,'identity',stats);
    W_boot(:,pi_index) = (Z-Pi)./(Pi.*(1-Pi));
end

save('temp/prel.mat') 


warning('off')

load('temp/prel.mat'); 

for numero = 1:2000

    m_index      = 1+rem(numero,size(method_list,2));
    method_index = method_list(m_index);
    K_n          = K_list(1+rem(1+(numero-m_index+1)/size(method_list,2),size(K_list,2)));       

    if method_index==2 | K_n<=4

        Y_d = sum(repmat(Y,[1,K_n])<=repmat(quantile(Y,(1:K_n)/K_n),[n,1]),2);

        if method_index == 1
            M = repmat(Y_d,[1,K_n])==repmat(1:K_n,[n,1]); 
        elseif method_index == 2
            M = repmat(K_n*T+Y_d,[1,2*K_n])==repmat(1:2*K_n,[n,1]);
        else
            M = repmat(2*K_n*R+K_n*T+Y_d,[1,4*K_n])==repmat(1:4*K_n,[n,1]);                
        end

        theta0 = UB;
        theta1 = LB;        
        for pi_index = 1:pi_grid

            itt_boot = (W_boot(:,pi_index)'*Y)/n;

            mean_WY = mean(W_boot(:,pi_index).*Y);
            mean_WM = mean(repmat(W_boot(:,pi_index),[1,size(M,2)]).*M);

            var_WY    = var(W_boot(:,pi_index).*Y);

            mean_WH   = [];    
            var_WH    = [];
            cov_WY_WH = [];
            for k = 1:size(M,2)
                M_one_list = combnk(1:size(M,2),k);
                for i_com = 1:size(M_one_list,1)
                    M_one = ismember(1:size(M,2),M_one_list(i_com,:))'-.5; 
                    mean_WH   = [mean_WH,mean_WM*M_one];
                    var_WH    = [var_WH,var(W_boot(:,pi_index).*(M*M_one))];
                    cov_WY_WH = [cov_WY_WH,(W_boot(:,pi_index).*Y)'*(W_boot(:,pi_index).*(M*M_one))/n-mean_WY*(mean_WM*M_one)];        
                end
            end    

            e_one = mean(epsilon_boot,1);
            e_WY  = epsilon_boot'*(W_boot(:,pi_index).*Y)/n;
            e_WM  = epsilon_boot'*(repmat(W_boot(:,pi_index),[1,size(M,2)]).*M)/n;        

            obj_fun = @(theta)func_Test(theta,M,mean_WY,mean_WH,var_WY,var_WH,cov_WY_WH,e_WY,e_WM,n,e_one,alpha,beta);

            if obj_fun(LB)>0        
                theta0 = min(theta0,fzero(obj_fun,[LB,itt_boot]));
            end
            if obj_fun(UB)>0
                theta1 = max(theta1,fzero(obj_fun,[itt_boot,UB]));
            end        
        end
        [theta0,theta1]
        save(strcat('temp/',num2str(method_index),'_',num2str(K_n),'.mat'),'theta0','theta1');   

    end
end

load('temp/prel.mat');

for method_index = method_list
    CI_theta = [];
    for K_n = K_list
        theta0 = UB;
        theta1 = LB;

        try 
            load(strcat('temp/',num2str(method_index),'_',num2str(K_n),'.mat'));
        catch err                      
        end

        CI_theta = [CI_theta;theta0,theta1];
    end
    CI_theta
    save(strcat('results/result',num2str(method_index),'.mat'),'CI_theta');
end